@@ -1,6 +1,4 @@
-#!/usr/bin/python
 # -*- coding: utf-8 -*-
-
 """
 =========================================================
 PCA example with Iris Data-set
@@ -12,15 +10,12 @@ See `here <https://en.wikipedia.org/wiki/Iris_flower_data_set>`_ for more
 information on this dataset.
 
 """
-print(__doc__)
-
 
 # Code source: Gaël Varoquaux
 # License: BSD 3 clause
 
 import numpy as np
 import matplotlib.pyplot as plt
-from mpl_toolkits.mplot3d import Axes3D
 
 
 from sklearn import decomposition
@@ -28,30 +23,34 @@ from sklearn import datasets
 
 np.random.seed(5)
 
-centers = [[1, 1], [-1, -1], [1, -1]]
 iris = datasets.load_iris()
 X = iris.data
 y = iris.target
 
 fig = plt.figure(1, figsize=(4, 3))
 plt.clf()
-ax = Axes3D(fig, rect=[0, 0, .95, 1], elev=48, azim=134)
+
+ax = fig.add_subplot(111, projection="3d", elev=48, azim=134)
+ax.set_position([0, 0, 0.95, 1])
+
 
 plt.cla()
 pca = decomposition.PCA(n_components=3)
 pca.fit(X)
 X = pca.transform(X)
 
-for name, label in [('Setosa', 0), ('Versicolour', 1), ('Virginica', 2)]:
-    ax.text3D(X[y == label, 0].mean(),
-              X[y == label, 1].mean() + 1.5,
-              X[y == label, 2].mean(), name,
-              horizontalalignment='center',
-              bbox=dict(alpha=.5, edgecolor='w', facecolor='w'))
+for name, label in [("Setosa", 0), ("Versicolour", 1), ("Virginica", 2)]:
+    ax.text3D(
+        X[y == label, 0].mean(),
+        X[y == label, 1].mean() + 1.5,
+        X[y == label, 2].mean(),
+        name,
+        horizontalalignment="center",
+        bbox=dict(alpha=0.5, edgecolor="w", facecolor="w"),
+    )
 # Reorder the labels to have colors matching the cluster results
-y = np.choose(y, [1, 2, 0]).astype(np.float)
-ax.scatter(X[:, 0], X[:, 1], X[:, 2], c=y, cmap=plt.cm.nipy_spectral,
-           edgecolor='k')
+y = np.choose(y, [1, 2, 0]).astype(float)
+ax.scatter(X[:, 0], X[:, 1], X[:, 2], c=y, cmap=plt.cm.nipy_spectral, edgecolor="k")
 
 ax.w_xaxis.set_ticklabels([])
 ax.w_yaxis.set_ticklabels([])
