@@ -9,8 +9,8 @@ the Digits dataset.
 This exercise is used in the :ref:`clf_tut` part of the
 :ref:`supervised_learning_tut` section of the
 :ref:`stat_learn_tut_index`.
+
 """
-print(__doc__)
 
 from sklearn import datasets, neighbors, linear_model
 
@@ -19,14 +19,16 @@ X_digits = X_digits / X_digits.max()
 
 n_samples = len(X_digits)
 
-X_train = X_digits[:int(.9 * n_samples)]
-y_train = y_digits[:int(.9 * n_samples)]
-X_test = X_digits[int(.9 * n_samples):]
-y_test = y_digits[int(.9 * n_samples):]
+X_train = X_digits[: int(0.9 * n_samples)]
+y_train = y_digits[: int(0.9 * n_samples)]
+X_test = X_digits[int(0.9 * n_samples) :]
+y_test = y_digits[int(0.9 * n_samples) :]
 
 knn = neighbors.KNeighborsClassifier()
 logistic = linear_model.LogisticRegression(max_iter=1000)
 
-print('KNN score: %f' % knn.fit(X_train, y_train).score(X_test, y_test))
-print('LogisticRegression score: %f'
-      % logistic.fit(X_train, y_train).score(X_test, y_test))
+print("KNN score: %f" % knn.fit(X_train, y_train).score(X_test, y_test))
+print(
+    "LogisticRegression score: %f"
+    % logistic.fit(X_train, y_train).score(X_test, y_test)
+)
